from __future__ import annotations

from abc import ABC, abstractmethod, abstractproperty
from typing import NewType

from result import Result

Url = NewType("Url", str)


class Backend(ABC):
    @abstractproperty
    def name(self) -> str:
        pass

    @abstractmethod
    def health_check(self) -> Result:
        pass

    @abstractmethod
    def post(self, content: str, url: str, *, tags: list[str] = []) -> Result[Url, str]:
        pass



from __future__ import annotations

import urllib.parse
from typing import TYPE_CHECKING

from atproto import Client, client_utils
from result import Err, Ok, Result

from autopost._backend.interface import Backend, Url

if TYPE_CHECKING:
    from autopost._config import BlueskyConfig


class Bluesky(Backend):
    def __init__(self, config: BlueskyConfig):
        self._config = config
        self._client = Client()
        self._client.login(self._config.username, self._config.password.get_secret_value())

    @property
    def name(self) -> str:
        return self._config.name

    def health_check(self) -> Result[None, str]:
        if self._client.me is not None:
            return Ok(None)
        else:
            return Err(f"{self._config.name} failed profile health check")

    def post(self, content: str, url: str, *, tags: list[str] = []) -> Result[Url, str]:
        post = client_utils.TextBuilder().text(f"{content} ").link(url, url)

        try:
            resp = self._client.send_post(post)

            # Bluesky returns a ridiculous at:// URI.
            # Mash it into something people actually use. These URIs look something like:
            #
            #   at://did:plc:SOMEJUNK/app.bsky.feed.post/IMPORTANTIDHERE
            #
            # ...where the only part we care about is "IMPORTANTIDHERE".
            post_id = urllib.parse.urlparse(resp.uri).path.split("/")[-1]
            return Ok(Url(f"https://bsky.app/profile/{self._config.username}/post/{post_id}"))
        except Exception as e:
            return Err(str(e))



from __future__ import annotations

from textwrap import dedent
from typing import TYPE_CHECKING

from mastodon import Mastodon as MastodonClient
from result import Err, Ok, Result

from autopost._backend.interface import Backend, Url

if TYPE_CHECKING:
    from autopost._config import MastodonConfig


class Mastodon(Backend):
    def __init__(self, config: MastodonConfig):
        self._config = config
        self._client = MastodonClient(
            api_base_url=self._config.server,
            client_id=self._config.client_key.get_secret_value(),
            client_secret=self._config.client_secret.get_secret_value(),
            access_token=self._config.access_token.get_secret_value(),
        )

    @property
    def name(self) -> str:
        return self._config.name

    def health_check(self) -> Result[None, str]:
        try:
            ok = self._client.instance_health()
            if ok:
                return Ok(None)
            return Err(f"{self._config.server} failed instance health check")
        except Exception as e:
            return Err(str(e))

    def post(self, content: str, url: str, *, tags: list[str] = []) -> Result[Url, str]:
        tags = [f"#{tag}" for tag in tags]

        status = dedent(
            f"""
            {content}

            {url}

            {' '.join(tags)}
            """
        )

        try:
            resp = self._client.status_post(status)
            return Ok(Url(resp["url"]))
        except Exception as e:
            return Err(str(e))



import os
from collections.abc import Iterator
from typing import Annotated, Literal

from pydantic import BaseModel, Field, SecretStr, StrictStr

from autopost._backend.interface import Backend


class Embedded(BaseModel):
    type_: Literal["Embedded"] = Field(alias="type")
    value: SecretStr

    def get_secret_value(self) -> str:
        return self.value.get_secret_value()


class Environment(BaseModel):
    type_: Literal["Environment"] = Field(alias="type")
    variable: StrictStr

    def get_secret_value(self) -> str:
        return os.environ[self.variable]


Credential = Annotated[Embedded | Environment, Field(discriminator="type_")]


class RedditConfig(BaseModel):
    type_: Literal["Reddit"] = Field(alias="type")
    name: str
    subreddit: str
    client_id: Credential
    client_secret: Credential
    username: str
    password: Credential


class TwitterConfig(BaseModel):
    type_: Literal["Twitter"] = Field(alias="type")
    name: str
    api_key: Credential
    api_key_secret: Credential
    access_token: Credential
    access_token_secret: Credential


class MastodonConfig(BaseModel):
    type_: Literal["Mastodon"] = Field(alias="type")
    name: str
    server: str
    client_secret: Credential
    client_key: Credential
    access_token: Credential


class BlueskyConfig(BaseModel):
    type_: Literal["Bluesky"] = Field(alias="type")
    name: str
    username: str
    password: Credential


BackendConfig = Annotated[
    RedditConfig | TwitterConfig | MastodonConfig | BlueskyConfig, Field(discriminator="type_")
]


class Config(BaseModel):
    backend_configs: list[BackendConfig] = Field(..., alias="backend")

    @property
    def backends(self) -> Iterator[Backend]:
        import autopost._backend as backend

        for config in self.backend_configs:
            klass = getattr(backend, config.type_)
            yield klass(config)
